--vars--
screenWidth, screenHeight = love.graphics.getDimensions()
infoPanelFont = love.graphics.setNewFont(20)
defaultFont = love.graphics.setNewFont(12)
points = {}
selectedPoint = nil
cellSize = 30
cellNum = 12
gridX = screenWidth / 2 - (cellSize * cellNum)
gridY = screenHeight / 2 - (cellSize * cellNum)
gridSize = cellSize * cellNum * 2
-- -- --

function love.load()
  love.graphics.setBackgroundColor({ 77, 77, 77, 255 })
  love.keyboard.setKeyRepeat(true)
end

function love.update(dt)
  local mouseX, mouseY = love.mouse.getPosition()
  for _, point in ipairs(points) do
    --check if is hovered and set it if is or unset it if isn't
    updateHoveredStatus(point, mouseX, mouseY)
    --check if is grabbed and set it if is or unset it if isn't
    updateGrabbedStatus(point)
  end
end

function love.draw()
  --draw grid
  make2DGrid(cellNum, cellSize)
  --draw points
  for _, point in ipairs(points) do
    --set color point, different if hovered
    if point.isHovered or point.isGrabbed or point == selectedPoint then
      love.graphics.setColor({ 180, 180, 22, 255 })
    else
      love.graphics.setColor({ 180, 22, 22, 255 })
    end
    --draw point
    love.graphics.circle("fill", point.x, point.y, 5)
    --draw (X,Y)coordinates text next to every point
    printPointInfo(point)
  end
  drawPointConnections()
  --draw info Panel
  drawInfoPanel(20, 20)
end

function love.keypressed(key)
  if (key == "escape") then
    love.event.quit()
  end
  if (key == 'kp+' and cellSize < screenHeight / 4) then
    ZoomView(4)
  end
  if (key == 'kp-' and cellSize > 20) then
    ZoomView(-4)
  end
  if (key == "f1") then
    local oldScreenWidth = screenWidth
    local oldScreenHeight = screenHeight

    if (not love.window.getFullscreen()) then
      love.window.setFullscreen(true)
      --resize points
      screenWidth, screenHeight = love.graphics.getDimensions()
      resizePoints(oldScreenWidth, oldScreenHeight)
    else
      love.window.setFullscreen(false)
      screenWidth, screenHeight = love.graphics.getDimensions()
      --resize points
      resizePoints(oldScreenWidth, oldScreenHeight)
    end
    updateGridSizeAndPointsPosition(0)
  end
end

function love.mousepressed(mouseX, mouseY, button)
  if button == 1 then
    addPoint(points, mouseX, mouseY, true)
  end
end

function love.mousemoved(mouseX, mouseY, dX, dY)
  for _, point in ipairs(points) do
    if point.isGrabbed then
      movePoint(point, mouseX, mouseY)
    end
  end
end

function make2DGrid(cellNum, cellSize)

  local lineColor = { 22, 22, 22, 255 }
  local x = screenWidth / 2 - cellNum * cellSize
  local y = screenHeight / 2 - cellNum * cellSize
  local width = screenWidth / 2 + cellNum * cellSize
  local height = screenHeight / 2 + cellNum * cellSize
  --line color
  love.graphics.setColor(lineColor)
  --draw central vertical line
  love.graphics.line(screenWidth / 2, y, screenWidth / 2, height)
  love.graphics.print(0, screenWidth / 2 + 2, screenHeight / 2 + 2)
  love.graphics.line(screenWidth / 2 - 1, y, screenWidth / 2 - 1, height)
  love.graphics.line(screenWidth / 2 + 1, y, screenWidth / 2 + 1, height)

  --draw central horizontal line
  love.graphics.line(x, screenHeight / 2, width, screenHeight / 2)
  love.graphics.line(x, screenHeight / 2 - 1, width, screenHeight / 2 - 1)
  love.graphics.line(x, screenHeight / 2 + 1, width, screenHeight / 2 + 1)
  --draw vertical lines
  for i = 1, cellNum do
    local positiveLine = screenWidth / 2 + (cellSize * i)
    local negativeLine = screenWidth / 2 - (cellSize * i)
    love.graphics.line(positiveLine, y, positiveLine, height)
    love.graphics.line(negativeLine, y, negativeLine, height)
    --numbers
    love.graphics.print(i, positiveLine + 2, screenHeight / 2 + 2)
    love.graphics.print(-i, negativeLine + 2, screenHeight / 2 + 2)
  end
  --draw horizontal lines
  for i = 1, cellNum do
    local positiveLine = screenHeight / 2 + (cellSize * i)
    local negativeLine = screenHeight / 2 - (cellSize * i)
    love.graphics.line(x, positiveLine, width, positiveLine)
    love.graphics.line(x, negativeLine, width, negativeLine)
    --numbers
    love.graphics.print(i, screenWidth / 2 + 2, positiveLine + 2)
    love.graphics.print(-i, screenWidth / 2 + 2, negativeLine + 2)
  end
end

function updateGridSizeAndPointsPosition(ammount)
  gridX = screenWidth / 2 - (cellSize * cellNum)
  gridY = screenHeight / 2 - (cellSize * cellNum)
  gridSize = cellSize * cellNum * 2

  for _, point in ipairs(points) do
    local cellX = math.abs((point.x - screenWidth / 2) / (cellSize - ammount))
    local cellY = math.abs((point.y - screenHeight / 2) / (cellSize - ammount))
    point.x = point.x < screenWidth / 2 and point.x - ammount * cellX or point.x + ammount * cellX
    point.y = point.y < screenHeight / 2 and point.y - ammount * math.abs(cellY) or point.y + ammount * cellY
    movePoint(point, point.x, point.y)
  end
end

function ZoomView(ammount)
  cellSize = cellSize + ammount
  updateGridSizeAndPointsPosition(ammount)
end

function addPoint(pointsTbl, mouseX, mouseY, bSnapToGrid) --if true, then snap function is active
  --sanitize points outside grid to be inside it
  mouseX, mouseY = sanitizeCoord(mouseX, mouseY)

  --snap points to grid if bool is true
  local newX = bSnapToGrid and round(((mouseX - gridX) / cellSize)) * cellSize + gridX or mouseX
  local newY = bSnapToGrid and round(((mouseY - gridY) / cellSize)) * cellSize + gridY or mouseY

  --check all point and return from function if we find another point at those coordinates
  for _, point in ipairs(points) do
    if checkOverlap(point, newX, newY) then return end
  end

  --if we find no overlap, add the new point
  pointsTbl[#pointsTbl + 1] = { ID = #pointsTbl + 1, x = newX, y = newY, isHovered = false, isGrabbed = false }
end

function sanitizeCoord(mouseX, mouseY)
  --sanitize points outside grid to be inside it
  if mouseX < gridX then mouseX = gridX
  elseif mouseX > gridX + gridSize then mouseX = gridX + gridSize
  end
  if mouseY < gridY then mouseY = gridY
  elseif mouseY > gridY + gridSize then mouseY = gridY + gridSize
  end
  return mouseX, mouseY
end

function updateGrabbedStatus(point)
  if point.isHovered then
    if love.mouse.isDown(1) then
      point.isGrabbed = true
      selectedPoint = point
    else
      point.isGrabbed = false
    end
  elseif --not Hovered
  point.isGrabbed == true and not love.mouse.isDown(1) then point.isGrabbed = false
  end
end

function movePoint(point, mouseX, mouseY)
  --sanitize points outside grid to be inside it
  mouseX, mouseY = sanitizeCoord(mouseX, mouseY)
  --new positions
  point.x = round(((mouseX - gridX) / cellSize)) * cellSize + gridX
  point.y = round(((mouseY - gridY) / cellSize)) * cellSize + gridY
end

function resizePoints(oldScreenWidth, oldScreenHeight)
  for _, point in ipairs(points) do
    point.x = point.x + ((screenWidth - oldScreenWidth) / 2)
    point.y = point.y + ((screenHeight - oldScreenHeight) / 2)
  end
end

function printPointInfo(point)
  --print coord
  local pointX = (point.x - screenWidth / 2) / cellSize
  local pointY = (point.y - screenHeight / 2) / cellSize
  love.graphics.setColor({ 180, 180, 180, 255 })
  love.graphics.print("(" .. pointX .. ", " .. pointY .. ")", point.x + 3, point.y + 3)
  --print point ID
  love.graphics.setFont(infoPanelFont)
  love.graphics.setColor({ 180, 25, 180, 255 })
  love.graphics.print(point.ID, point.x - 9, point.y + 4)
end

function drawInfoPanel(infoPanelX, infoPanelY)
  local infoPanelWidth = 320
  local infoPanelHeight = screenHeight - infoPanelY * 2
  --draw panel
  love.graphics.setColor({ 33, 33, 33, 220 })
  love.graphics.rectangle("fill", infoPanelX, infoPanelY, infoPanelWidth, infoPanelHeight)
  --draw selected info
  local lineX = infoPanelX + 10
  local lineY = infoPanelX + 10
  --make sure selected is not nil
  if selectedPoint then
    love.graphics.setFont(infoPanelFont)
    love.graphics.setColor({ 144, 144, 144, 230 })
    love.graphics.print("Selected Point Info", lineX, lineY)
    love.graphics.print("Point N: " .. selectedPoint.ID, lineX * 2, lineY * 3)

    --if we have more than 1 point, add info about distance from the next
    if #points > 1 then
      local next = (selectedPoint.ID % #points) + 1
      local distance = findDistance(selectedPoint, points[next])
      love.graphics.print("Distance from P " .. points[next].ID .. ": " .. string.format("%.3f", distance), lineX * 2, lineY * 4)
    end
  end
  love.graphics.setFont(defaultFont)
end

function drawPointConnections()
  love.graphics.setColor({ 34, 120, 220, 255 })
  for key, point in ipairs(points) do
    local next = (key % #points) + 1
    love.graphics.line(point.x, point.y, points[next].x, points[next].y)
  end
end

function round(num)

  local resoult
  if num - math.floor(num) <= 0.5 then
    resoult = math.floor(num)
  else
    resoult = math.ceil(num)
  end
  return resoult
end

function updateHoveredStatus(point, mouseX, mouseY)
  if checkOverlap(point, mouseX, mouseY) then
    point.isHovered = true
  else
    point.isHovered = false
  end
end

function checkOverlap(point, mouseX, mouseY)
  local radius = 6
  local x = point.x - radius
  local y = point.y - radius
  local width = point.x + radius
  local height = point.y + radius
  if mouseX > x and mouseX < width and
     mouseY > y and mouseY < height then
    return true
  else
    return false
  end
  return false
end

function findDistance(point1, point2)
  local point1X = (point1.x - screenWidth / 2) / cellSize
  local point1Y = (point1.y - screenHeight / 2) / cellSize
  local point2X = (point2.x - screenWidth / 2) / cellSize
  local point2Y = (point2.y - screenHeight / 2) / cellSize
  local deltaX = math.pow(point1X - point2X, 2)
  local deltaY = math.pow(point1Y - point2Y, 2)
  local distance = math.sqrt(deltaX + deltaY)
  return distance
end